@@ -36,6 +36,7 @@ struct amdgpu_pixmap {
 
 	struct amdgpu_buffer *bo;
 	struct drmmode_fb *fb;
+	Bool fb_failed;
 
 	/* GEM handle for pixmaps shared via DRI2/3 */
 	Bool handle_valid;
@@ -62,7 +63,7 @@ static inline Bool amdgpu_set_pixmap_bo(PixmapPtr pPix, struct amdgpu_buffer *bo
 	struct amdgpu_pixmap *priv;
 
 	priv = amdgpu_get_pixmap_private(pPix);
-	if (priv == NULL && bo == NULL)
+	if (!priv && !bo)
 		return TRUE;
 
 	if (priv) {
@@ -104,8 +105,8 @@ static inline struct amdgpu_buffer *amdgpu_get_pixmap_bo(PixmapPtr pPix)
 }
 
 static inline struct drmmode_fb*
-amdgpu_fb_create(int drm_fd, uint32_t width, uint32_t height, uint8_t depth,
-		 uint8_t bpp, uint32_t pitch, uint32_t handle)
+amdgpu_fb_create(ScrnInfoPtr scrn, int drm_fd, uint32_t width, uint32_t height,
+		 uint32_t pitch, uint32_t handle)
 {
 	struct drmmode_fb *fb  = malloc(sizeof(*fb));
 
@@ -113,8 +114,8 @@ amdgpu_fb_create(int drm_fd, uint32_t width, uint32_t height, uint8_t depth,
 		return NULL;
 
 	fb->refcnt = 1;
-	if (drmModeAddFB(drm_fd, width, height, depth, bpp, pitch, handle,
-			 &fb->handle) == 0)
+	if (drmModeAddFB(drm_fd, width, height, scrn->depth, scrn->bitsPerPixel,
+			 pitch, handle, &fb->handle) == 0)
 		return fb;
 
 	free(fb);
@@ -143,22 +144,22 @@ static inline struct drmmode_fb*
 amdgpu_pixmap_get_fb(PixmapPtr pix)
 {
 	struct drmmode_fb **fb_ptr = amdgpu_pixmap_get_fb_ptr(pix);
+	uint32_t handle;
 
-	if (!fb_ptr)
-		return NULL;
+	if (fb_ptr && *fb_ptr)
+		return *fb_ptr;
+	
+	if (amdgpu_pixmap_get_handle(pix, &handle)) {
+		ScrnInfoPtr scrn = xf86ScreenToScrn(pix->drawable.pScreen);
+		AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
 
-	if (!*fb_ptr) {
-		uint32_t handle;
+		if (!fb_ptr)
+			fb_ptr = amdgpu_pixmap_get_fb_ptr(pix);
 
-		if (amdgpu_pixmap_get_handle(pix, &handle)) {
-			ScrnInfoPtr scrn = xf86ScreenToScrn(pix->drawable.pScreen);
-			AMDGPUEntPtr pAMDGPUEnt = AMDGPUEntPriv(scrn);
-
-			*fb_ptr = amdgpu_fb_create(pAMDGPUEnt->fd, pix->drawable.width,
-						   pix->drawable.height, pix->drawable.depth,
-						   pix->drawable.bitsPerPixel, pix->devKind,
-						   handle);
-		}
+		*fb_ptr = amdgpu_fb_create(scrn, pAMDGPUEnt->fd,
+					   pix->drawable.width,
+					   pix->drawable.height, pix->devKind,
+					   handle);
 	}
 
 	return *fb_ptr;
